import { composeWithDevTools } from '@redux-devtools/extension';
import { applyMiddleware, combineReducers, compose, createStore, } from 'redux';
import createSagaMiddleware from 'redux-saga';
import { clearStore } from 'redux-store-generator';
export class StoreBuilder {
    constructor(name) {
        this.definition = {
            name: '',
            initialState: {},
            reducers: {},
            middlewares: [],
            enhancers: [],
            sagas: [],
            sagasContext: {},
            enableDevtoolsExtension: false,
            sagaMonitor: null,
        };
        this.preMiddlewares = [];
        this.postMiddlewares = [];
        this.preBuildHooks = [];
        this.postBuildHooks = [];
        this.autoClear = false;
        this.definition.name = name;
    }
    withInitialState(initialState) {
        this.definition.initialState = {
            ...this.definition.initialState,
            ...initialState,
        };
        return this;
    }
    withPreBuildHook(callback) {
        this.preBuildHooks.push(callback);
        return this;
    }
    withPostBuildHook(callback) {
        this.postBuildHooks.push(callback);
        return this;
    }
    withReducers(reducers) {
        this.definition.reducers = {
            ...this.definition.reducers,
            ...reducers,
        };
        return this;
    }
    withMiddlewares(middlewares) {
        const extraMiddlewares = Array.isArray(middlewares)
            ? middlewares
            : [middlewares];
        this.definition.middlewares = [
            ...this.definition.middlewares,
            ...extraMiddlewares,
        ];
        return this;
    }
    withEnhancers(enhancers) {
        const extraEnhancers = Array.isArray(enhancers) ? enhancers : [enhancers];
        this.definition.enhancers = [
            ...this.definition.enhancers,
            ...extraEnhancers,
        ];
        return this;
    }
    withPreMiddlewares(...middlewares) {
        this.preMiddlewares = middlewares;
        return this;
    }
    withPostMiddlewares(...middlewares) {
        this.postMiddlewares = middlewares;
        return this;
    }
    withSagas(...sagas) {
        this.definition.sagas = [...this.definition.sagas, ...sagas];
        return this;
    }
    withSagaMonitor(sagaMonitor) {
        this.definition.sagaMonitor = sagaMonitor;
        return this;
    }
    withSagaContext(context) {
        this.definition.sagasContext = {
            ...this.definition.sagasContext,
            ...context,
        };
        return this;
    }
    clearSagas() {
        this.definition.sagas = [];
        return this;
    }
    withDevtoolsExtensions(enable) {
        this.definition.enableDevtoolsExtension = enable === true;
        return this;
    }
    withAutoClear() {
        this.autoClear = true;
        return this;
    }
    getSagaMiddleware() {
        return this.sagaMiddleware;
    }
    clear(store) {
        clearStore(store);
    }
    build() {
        // add the saga middleware
        this.preBuildHooks.forEach((callback) => {
            callback(store);
        });
        if (this.definition.sagas.length > 0) {
            this.sagaMiddleware = createSagaMiddleware({
                context: {
                    ...this.definition.sagasContext,
                },
                sagaMonitor: this.definition.sagaMonitor,
            });
            this.withMiddlewares(this.sagaMiddleware);
        }
        const rootReducer = combineReducers(this.definition.reducers);
        const middlewareEnhancer = applyMiddleware(...this.preMiddlewares, ...this.definition.middlewares, ...this.postMiddlewares);
        const composeMethod = this.definition.enableDevtoolsExtension
            ? composeWithDevTools
            : compose;
        const composedEnhancers = composeMethod(middlewareEnhancer, ...this.definition.enhancers);
        const store = createStore(rootReducer, this.definition.initialState, composedEnhancers);
        this.postBuildHooks.forEach((callback) => {
            callback(store);
        });
        // running sagas
        this.definition.sagas.forEach((saga) => {
            this.sagaMiddleware.run(saga);
        });
        if (this.autoClear) {
            this.clear(store);
        }
        return store;
    }
}
